@@ -34,8 +34,8 @@
 
 #include "xnconfig.h"
 
-#define LIBRARY_VERSION_MAJOR		4
-#define LIBRARY_VERSION_MINOR		0
+#define LIBRARY_VERSION_MAJOR		9
+#define LIBRARY_VERSION_MINOR		3
 #define OPTIONS_DELIMETER		" "
 
 static const char *log_levels[] =	{"Error", "Warning", "Log", "Debug", "Trace"};
@@ -48,18 +48,25 @@ static const char *option_names[] = 	{
 						"ConsonantLetter", "NoFirstLetter", "SetAutoApp", "SetManualApp", "GrabMouse",
 						"EducationMode", "Version", "LayoutRememberMode", "SaveSelectionMode",
 						"DefaultXkbGroup", "AddSound", "PlaySounds", "SendDelay", "LayoutRememberModeForApp",
-						"DrawFlagApp", "AddFlagPixmap", "SaveLog"
+						"DrawFlagApp", "AddFlagPixmap", "SaveLog", "ReplaceAbbreviation",
+						"ReplaceAbbreviationIgnoreLayout", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
+						"FlushBufferWhenPressEnter", "DontProcessWhenPressEnter", "AddAction",
+						"ShowOSD", "AddOSD", "FontOSD"
 					};
 static const char *action_names[] =	{
 						"ChangeWord", "ChangeString", "ChangeMode",
 						"ChangeSelected", "TranslitSelected", "ChangecaseSelected",
-						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4"
+						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
+						"ReplaceAbbreviation"
 					};
 static const char *sound_names[] =	{
+						"XneurStart", "XneurReload", "XneurStop",
 						"PressKeyLayout1", "PressKeyLayout2", "PressKeyLayout3", "PressKeyLayout4",
 						"EnableLayout1", "EnableLayout2", "EnableLayout3", "EnableLayout4",
-						"AutomaticChangeWord", "ManualChangeWord", "ChangeString", 
-						"ChangeSelected", "TranslitSelected", "ChangecaseSelected"
+						"AutomaticChangeWord", "ManualChangeWord", "ChangeString",
+						"ChangeSelected", "TranslitSelected", "ChangecaseSelected",
+						"ReplaceAbbreviation", "CorrectIncidentalCaps", "CorrectTwoCapitalLetter",
+						"ExecuteUserAction"
 					};
 
 static int load_lang = -1;
@@ -74,6 +81,7 @@ static char* get_word(char **string)
 
 #define get_option_index(options, option) \
 	get_option_index_size(options, option, sizeof(options) / sizeof(options[0]));
+
 static int get_option_index_size(const char *options[], char *option, int options_count)
 {
 	for (int i = 0; i < options_count; i++)
@@ -105,14 +113,17 @@ static void parse_line(struct _xneur_config *p, char *line)
 	int index = get_option_index(option_names, option);
 	if (index == -1)
 	{
-		log_message(WARNING, "Unrecognized option \"%s\" detected", option);
+		log_message(WARNING, _("Unrecognized option \"%s\" detected"), option);
 		return;
 	}
 
+	char *full_string = strdup(line);
+
 	char *param = get_word(&line);
 	if (param == NULL)
 	{
-		log_message(WARNING, "Param mismatch for option %s", option);
+		free(full_string);
+		log_message(WARNING, _("Param mismatch for option %s"), option);
 		return;
 	}
 
@@ -123,16 +134,17 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int manual = get_option_index(bool_names, param);
 			if (manual == -1)
 			{
-				log_message(WARNING, "Invalid value for manual mode specified");
+				log_message(WARNING, _("Invalid value for manual mode specified"));
 				break;
 			}
 
 			p->set_manual_mode(p, manual);
+			p->manual_mode = manual;
 			break;
 		}
 		case 1: // Get Applications Names
 		{
-			p->excluded_apps->add(p->excluded_apps, param);
+			p->excluded_apps->add(p->excluded_apps, full_string);
 			break;
 		}
 		case 2: // Get Keyboard Binds
@@ -140,7 +152,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int action = get_option_index(action_names, param);
 			if (action == -1)
 			{
-				log_message(WARNING, "Invalid value for action name specified");
+				log_message(WARNING, _("Invalid value for action name specified"));
 				break;
 			}
 
@@ -167,7 +179,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(log_levels, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for log level specified");
+				log_message(WARNING, _("Invalid value for log level specified"));
 				break;
 			}
 
@@ -182,7 +194,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 
 			if (dir == NULL || group == NULL)
 			{
-				log_message(ERROR, "Argument number mismatch for AddLanguage option");
+				log_message(ERROR, _("Argument number mismatch for AddLanguage option"));
 				break;
 			}
 
@@ -206,12 +218,12 @@ static void parse_line(struct _xneur_config *p, char *line)
 		}
 		case 8: // Get Auto Processing Applications
 		{
-			p->auto_apps->add(p->auto_apps, param);
+			p->auto_apps->add(p->auto_apps, full_string);
 			break;
 		}
 		case 9: // Get Manual Processing Applications
 		{
-			p->manual_apps->add(p->manual_apps, param);
+			p->manual_apps->add(p->manual_apps, full_string);
 			break;
 		}
 		case 10: // Get Mouse Grab Mode
@@ -219,7 +231,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for mouse grab mode specified");
+				log_message(WARNING, _("Invalid value for mouse grab mode specified"));
 				break;
 			}
 
@@ -231,7 +243,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for education mode specified");
+				log_message(WARNING, _("Invalid value for education mode specified"));
 				break;
 			}
 
@@ -248,7 +260,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for remember layout mode specified");
+				log_message(WARNING, _("Invalid value for remember layout mode specified"));
 				break;
 			}
 
@@ -260,7 +272,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for save selection mode specified");
+				log_message(WARNING, _("Invalid value for save selection mode specified"));
 				break;
 			}
 
@@ -277,11 +289,21 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int sound = get_option_index(sound_names, param);
 			if (sound == -1)
 			{
-				log_message(WARNING, "Invalid value for sound action name specified");
+				log_message(WARNING, _("Invalid value for sound action name specified"));
 				break;
 			}
 
-			p->sounds[sound].file = strdup(get_word(&line));
+			if (line == NULL)
+				break;
+			
+			char *file = strdup(get_word(&line));
+			if (strlen(file) != 0)
+			{
+				p->sounds[sound].file = get_file_path_name(SOUNDDIR, file);
+			}
+			if (file != NULL)
+				free(file);
+			
 			break;
 		}
 		case 17: // Play Sound
@@ -289,7 +311,7 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for play sounds mode specified");
+				log_message(WARNING, _("Invalid value for play sounds mode specified"));
 				break;
 			}
 
@@ -301,19 +323,19 @@ static void parse_line(struct _xneur_config *p, char *line)
 			p->send_delay = atoi(param);
 			if (p->send_delay < 0 || p->send_delay > 50)
 			{
-				log_message(WARNING, "Send delay must be between 0 and 50");
+				log_message(WARNING, _("Send delay must be between 0 and 50"));
 				p->send_delay = 0;
 			}
 			break;
 		}
 		case 19: // layout remember for each application
 		{
-			p->layout_remember_apps->add(p->layout_remember_apps, param);
+			p->layout_remember_apps->add(p->layout_remember_apps, full_string);
 			break;
 		}
 		case 20: // Get Draw Flag Applications
 		{
-			p->draw_flag_apps->add(p->draw_flag_apps, param);
+			p->draw_flag_apps->add(p->draw_flag_apps, full_string);
 			break;
 		}
 		case 21: // Flags
@@ -321,11 +343,19 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int flag = get_option_index(flag_names, param);
 			if (flag == -1)
 			{
-				log_message(WARNING, "Invalid value for flag layout name specified");
+				log_message(WARNING, _("Invalid value for flag layout name specified"));
 				break;
 			}
 
-			p->flags[flag].file = strdup(get_word(&line));
+			if (line == NULL)
+				break;
+			
+			char *file = strdup(get_word(&line));
+			if (strlen(file) != 0)
+			{
+				p->flags[flag].file = get_file_path_name(PIXMAPDIR, file);
+				free(file);
+			}
 			break;
 		}
 		case 22: // Save Keyboard Log
@@ -333,22 +363,155 @@ static void parse_line(struct _xneur_config *p, char *line)
 			int index = get_option_index(bool_names, param);
 			if (index == -1)
 			{
-				log_message(WARNING, "Invalid value for save keyboard log mode specified");
+				log_message(WARNING, _("Invalid value for save keyboard log mode specified"));
 				break;
 			}
 
 			p->save_keyboard_log = index;
 			break;
 		}
+		case 23: // Get Words for Replacing
+		{
+			p->abbreviations->add(p->abbreviations, full_string);
+			break;
+		}
+		case 24: // Ignore keyboard layout for abbreviations
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for ignore keyboard layout for abbreviations mode specified"));
+				break;
+			}
+
+			p->abbr_ignore_layout = index;
+			break;
+		}
+		case 25: // Change iNCIDENTAL CapsLock Mode
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for change iNCIDENTAL CapsLock mode specified"));
+				break;
+			}
+
+			p->correct_incidental_caps = index;
+			break;
+		}
+		case 26: // Change two CApital letter Mode
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for change two CApital letter mode specified"));
+				break;
+			}
+
+			p->correct_two_capital_letter = index;
+			break;
+		}
+		case 27: // Flush internal buffer when pressed Enter Mode
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for flush internal buffer when pressed Enter mode specified"));
+				break;
+			}
+
+			p->flush_buffer_when_press_enter = index;
+			break;
+		}
+		case 28: // Don't process word when pressed Enter Mode
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for don't processing word when pressed Enter mode specified"));
+				break;
+			}
+
+			p->dont_process_when_press_enter = index;
+			break;
+		}
+		case 29: // User actions
+		{
+			int action = p->actions->action_command->data_count;
+			p->actions->action_hotkey =  (struct _xneur_hotkey *) realloc(p->actions->action_hotkey, (action + 1) * sizeof(struct _xneur_hotkey));
+			p->actions->action_hotkey[action].key = NULL;
+			p->actions->action_hotkey[action].modifiers = 0;
+			
+			while (TRUE)
+			{
+				
+				if (param == NULL)
+					break;
+
+				if (param[0] == '\0')
+					continue;
+				
+				int index = get_option_index(modifier_names, param);
+
+				if (index == -1)
+				{
+					if (param != NULL)
+						p->actions->action_hotkey[action].key = strdup(param);
+					if (line != NULL)
+						p->actions->action_command->add_last(p->actions->action_command, line);
+					break;
+				}	
+				else
+					p->actions->action_hotkey[action].modifiers |= (1 << index);
+
+				param = get_word(&line);
+			}
+			break;
+		}
+		case 30: // Show OSD
+		{
+			int index = get_option_index(bool_names, param);
+			if (index == -1)
+			{
+				log_message(WARNING, _("Invalid value for show OSD mode specified"));
+				break;
+			}
+
+			p->show_osd = index;
+			break;
+		}
+		case 31: // OSDs
+		{
+			int osd = get_option_index(sound_names, param);
+			if (osd == -1)
+			{
+				log_message(WARNING, _("Invalid value for OSD action name specified"));
+				break;
+			}
+
+			if (line == NULL)
+				break;
+			
+			if (strlen(line) != 0)
+				p->osds[osd].file = strdup(line);
+			
+			break;
+		}
+		case 32: // Get Initial Xkb Group for all new windows
+		{
+			p->osd_font = strdup(param);
+			break;
+		}
 	}
+	free(full_string);
 }
 
 static int parse_config_file(struct _xneur_config *p, const char *dir_name, const char *file_name)
-{	
+{
 	struct _list_char *list = load_list(dir_name, file_name, FALSE);
 	if (list == NULL)
 	{
-		log_message(ERROR, "Can't find config file %s", file_name);
+		log_message(ERROR, _("Can't find config file %s"), file_name);
 		return FALSE;
 	}
 
@@ -373,12 +536,15 @@ static int check_memory_attached(struct _xneur_config *p)
 
 static void free_structures(struct _xneur_config *p)
 {
+	int total_user_actions = p->actions->action_command->data_count;
+	
 	p->window_layouts->uninit(p->window_layouts);
 	p->manual_apps->uninit(p->manual_apps);
 	p->auto_apps->uninit(p->auto_apps);
 	p->layout_remember_apps->uninit(p->layout_remember_apps);
 	p->excluded_apps->uninit(p->excluded_apps);
 	p->draw_flag_apps->uninit(p->draw_flag_apps);
+	p->actions->action_command->uninit(p->actions->action_command);
 	
 	if (p->version != NULL)
 	{
@@ -424,26 +590,40 @@ static void free_structures(struct _xneur_config *p)
 		p->languages = NULL;
 	}
 
+	for (int user_action = 0; user_action < total_user_actions; user_action++)
+	{
+		if (p->actions->action_hotkey[user_action].key != NULL)
+			free(p->actions->action_hotkey[user_action].key);
+	}
+		
 	for (int action = 0; action < MAX_HOTKEYS; action++)
 	{
 		if (p->hotkeys[action].key != NULL)
 			free(p->hotkeys[action].key);
 	}
-	
+
 	for (int sound = 0; sound < MAX_SOUNDS; sound++)
 	{
 		if (p->sounds[sound].file != NULL)
 			free(p->sounds[sound].file);
 	}
 
+	for (int osd = 0; osd < MAX_OSDS; osd++)
+	{
+		if (p->osds[osd].file != NULL)
+			free(p->osds[osd].file);
+	}
+	
 	for (int flag = 0; flag < MAX_FLAGS; flag++)
 	{
 		if (p->flags[flag].file != NULL)
 			free(p->flags[flag].file);
 	}
-	
+
+	bzero(p->actions->action_hotkey, total_user_actions * sizeof(struct _xneur_hotkey));
 	bzero(p->hotkeys, MAX_HOTKEYS * sizeof(struct _xneur_hotkey));
 	bzero(p->sounds, MAX_SOUNDS * sizeof(struct _xneur_file));
+	bzero(p->osds, MAX_OSDS * sizeof(struct _xneur_file));
 	bzero(p->flags, MAX_FLAGS * sizeof(struct _xneur_file));
 }
 
@@ -504,7 +684,7 @@ static int xneur_config_load(struct _xneur_config *p)
 
 	if (p->total_languages == 0)
 	{
-		log_message(ERROR, "No languages specified in config file");
+		log_message(ERROR, _("No languages specified in config file"));
 		return FALSE;
 	}
 
@@ -516,28 +696,28 @@ static int xneur_config_load(struct _xneur_config *p)
 		p->languages[lang].dicts = load_list(lang_dir, DICT_NAME, TRUE);
 		if (p->languages[lang].dicts == NULL)
 		{
-			log_message(ERROR, "Can't find dictionary file for %s language", lang_name);
+			log_message(ERROR, _("Can't find dictionary file for %s language"), lang_name);
 			return FALSE;
 		}
 
 		p->languages[lang].protos = load_list(lang_dir, PROTO_NAME, TRUE);
 		if (p->languages[lang].protos == NULL)
 		{
-			log_message(ERROR, "Can't find protos file for %s language", lang_name);
+			log_message(ERROR, _("Can't find protos file for %s language"), lang_name);
 			return FALSE;
 		}
 
 		p->languages[lang].big_protos = load_list(lang_dir, BIG_PROTO_NAME, TRUE);
 		if (p->languages[lang].big_protos == NULL)
 		{
-			log_message(ERROR, "Can't find big protos file for %s language", lang_name);
+			log_message(ERROR, _("Can't find big protos file for %s language"), lang_name);
 			return FALSE;
 		}
 
 		p->languages[lang].regexp = load_list(lang_dir, REGEXP_NAME, TRUE);
 		if (p->languages[lang].regexp == NULL)
 		{
-			log_message(ERROR, "Can't find regexp file for %s language", lang_name);
+			log_message(ERROR, _("Can't find regexp file for %s language"), lang_name);
 			return FALSE;
 		}
 
@@ -547,7 +727,7 @@ static int xneur_config_load(struct _xneur_config *p)
 		if (!parse_config_file(p, lang_dir, LANGDEF_NAME))
 			return FALSE;
 	}
-	
+
 	return TRUE;
 }
 
@@ -561,18 +741,20 @@ static void xneur_config_clear(struct _xneur_config *p)
 	p->layout_remember_apps	= list_char_init();
 	p->excluded_apps	= list_char_init();
 	p->draw_flag_apps	= list_char_init();
+	p->abbreviations	= list_char_init();
+	p->actions->action_command = list_char_init();
 }
 
 static int xneur_config_save(struct _xneur_config *p)
 {
  	char *config_file_path_name = get_home_file_path_name(NULL, CONFIG_NAME);
 
-	log_message(LOG, "Saving main config to %s", config_file_path_name);
+	log_message(LOG, _("Saving main config to %s"), config_file_path_name);
 
 	FILE *stream = fopen(config_file_path_name, "w");
 	if (stream == NULL)
 	{
-		log_message(ERROR, "Can't create file %s", config_file_path_name);
+		log_message(ERROR, _("Can't create file %s"), config_file_path_name);
 		free(config_file_path_name);
 		return FALSE;
 	}
@@ -582,7 +764,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	fprintf(stream, "# It's a X Neural Switcher configuration file by XNeur\n# All values writted XNeur\n\n");
 
 	fprintf(stream, "# Config version\nVersion %s\n\n", VERSION);
-	fprintf(stream, "# Work in manual mode\nManualMode %s\n\n", p->get_bool_name(p->is_manual_mode(p)));
+	fprintf(stream, "# Work in manual mode\nManualMode %s\n\n", p->get_bool_name(p->manual_mode));
 
 	fprintf(stream, "# Level of messages program will write to output\n");
 	fprintf(stream, "#LogLevel Error\n");
@@ -601,7 +783,7 @@ static int xneur_config_save(struct _xneur_config *p)
 
 	fprintf(stream, "# Define initial keyboard layout for all new applications\n");
 	fprintf(stream, "DefaultXkbGroup %d\n\n", p->default_group);
-	
+
 	fprintf(stream, "# Add Applications names to exclude it from procces with xneur\n");
 	fprintf(stream, "# Xneur will not process the input for this applications\n");
 	fprintf(stream, "# Example:\n");
@@ -642,21 +824,52 @@ static int xneur_config_save(struct _xneur_config *p)
 	}
 	fprintf(stream, "\n");
 
+	fprintf(stream, "# This option add user action when pressed key bind\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#AddAction Control Alt f firefox\n");
+	for (int action = 0; action < p->actions->action_command->data_count; action++)
+	{
+		fprintf(stream, "AddAction ");
+
+		const int total_modifiers = sizeof(modifier_names) / sizeof(modifier_names[0]);
+		for (int i = 0; i < total_modifiers; i++)
+		{
+			if (p->actions->action_hotkey[action].modifiers & (1 << i))
+				fprintf(stream, "%s ", modifier_names[i]);
+		}
+
+		fprintf(stream, "%s %s\n", p->actions->action_hotkey[action].key
+								, p->actions->action_command->data[action].string);
+	}
+	fprintf(stream, "\n");
+			
+	fprintf(stream, "# Word Replacing\n# Ignore keyboard layout for abbreviations list\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#ReplaceAbbreviationIgnoreLayout No\n");
+	fprintf(stream, "ReplaceAbbreviationIgnoreLayout %s\n\n", p->get_bool_name(p->abbr_ignore_layout));
+			
+	fprintf(stream, "# Abbreviations list\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#ReplaceAbbreviation xneur X Neural Switcher\n");
+	for (int words = 0; words < p->abbreviations->data_count; words++)
+		fprintf(stream, "ReplaceAbbreviation %s\n", p->abbreviations->data[words].string);
+	fprintf(stream, "\n");
+
 	fprintf(stream, "# This option enable or disable sound playing\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#PlaySounds No\n");
 	fprintf(stream, "PlaySounds %s\n\n", p->get_bool_name(p->play_sounds));
-	
+
 	fprintf(stream, "# Binds sounds for some actions\n");
 	for (int sound = 0; sound < MAX_SOUNDS; sound++)
 	{
 		if (p->sounds[sound].file == NULL)
-			fprintf(stream, "AddSound %s\n", sound_names[sound]);
+			fprintf(stream, "AddSound %s \n", sound_names[sound]);
 		else
 			fprintf(stream, "AddSound %s %s\n", sound_names[sound], p->sounds[sound].file);
 	}
 	fprintf(stream, "\n");
-	
+
 	fprintf(stream, "# This option enable or disable mouse processing\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#GrabMouse Yes\n");
@@ -679,7 +892,7 @@ static int xneur_config_save(struct _xneur_config *p)
 	for (int i = 0; i < p->layout_remember_apps->data_count; i++)
 		fprintf(stream, "LayoutRememberModeForApp %s\n", p->layout_remember_apps->data[i].string);
 	fprintf(stream, "\n");
-	
+
 	fprintf(stream, "# This option enable or disable saving selection text\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#SaveSelectionMode No\n");
@@ -687,10 +900,10 @@ static int xneur_config_save(struct _xneur_config *p)
 
 	fprintf(stream, "# This option define delay before sendind events to application (in milliseconds between 0 to 50).\n");
 	fprintf(stream, "SendDelay %d\n\n", p->send_delay);
-	
-	fprintf(stream, "# Binds pixmaps for some layouts (pixmap only in xpm format)\n");
+
+	fprintf(stream, "# Binds pixmaps for some layouts\n");
 	fprintf(stream, "# Example:\n");
-	fprintf(stream, "#AddFlagPixmap <Layout1Flag|Layout2Flag|Layout3Flag|Layout4Flag> English.xpm\n");			
+	fprintf(stream, "#AddFlagPixmap <Layout1Flag|Layout2Flag|Layout3Flag|Layout4Flag> English.png\n");
 	for (int flag = 0; flag < MAX_FLAGS; flag++)
 	{
 		if (p->flags[flag].file == NULL)
@@ -699,19 +912,59 @@ static int xneur_config_save(struct _xneur_config *p)
 		fprintf(stream, "AddFlagPixmap %s %s\n", flag_names[flag], p->flags[flag].file);
 	}
 	fprintf(stream, "\n");
-			
+
 	fprintf(stream, "# Add Applications names to draw flag in window\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#DrawFlagApp Gedit\n");
 	for (int i = 0; i < p->draw_flag_apps->data_count; i++)
 		fprintf(stream, "DrawFlagApp %s\n", p->draw_flag_apps->data[i].string);
 	fprintf(stream, "\n");
-			
+
 	fprintf(stream, "# This option enable or disable logging keyboard\n");
 	fprintf(stream, "# Example:\n");
 	fprintf(stream, "#SaveLog No\n");
 	fprintf(stream, "SaveLog %s\n\n", p->get_bool_name(p->save_keyboard_log));
+
+	fprintf(stream, "# This option enable or disable correction of iNCIDENTAL CapsLock\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectIncidentalCaps Yes\n");
+	fprintf(stream, "CorrectIncidentalCaps %s\n\n", p->get_bool_name(p->correct_incidental_caps));
+
+	fprintf(stream, "# This option enable or disable correction of two CApital letter\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#CorrectTwoCapitalLetter Yes\n");
+	fprintf(stream, "CorrectTwoCapitalLetter %s\n\n", p->get_bool_name(p->correct_two_capital_letter));
 			
+	fprintf(stream, "# This option enable or disable flushing internal buffer when pressed Enter or Tab\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#FlushBufferWhenPressEnter Yes\n");
+	fprintf(stream, "FlushBufferWhenPressEnter %s\n\n", p->get_bool_name(p->flush_buffer_when_press_enter));
+
+	fprintf(stream, "# This option disable or enable processing word when pressed Enter or Tab\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#DontProcessWhenPressEnter Yes\n");
+	fprintf(stream, "DontProcessWhenPressEnter %s\n\n", p->get_bool_name(p->dont_process_when_press_enter));
+			
+	fprintf(stream, "# This option disable or enable show OSD\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#ShowOSD Yes\n");
+	fprintf(stream, "ShowOSD %s\n\n", p->get_bool_name(p->show_osd));
+
+	fprintf(stream, "# This option set font for OSD\n");
+	fprintf(stream, "# Example:\n");
+	fprintf(stream, "#FontOSD -*-*-*-*-*-*-32-*-*-*-*-*-*-u\n");
+	fprintf(stream, "FontOSD %s\n\n", p->osd_font);
+			
+	fprintf(stream, "# Binds OSDs for some actions\n");
+	for (int osd= 0; osd < MAX_OSDS; osd++)
+	{
+		if (p->osds[osd].file == NULL)
+			fprintf(stream, "AddOSD %s \n", sound_names[osd]);
+		else
+			fprintf(stream, "AddOSD %s %s\n", sound_names[osd], p->osds[osd].file);
+	}
+	fprintf(stream, "\n");
+
 	fprintf(stream, "# That's all\n");
 
 	fclose(stream);
@@ -724,13 +977,13 @@ static int xneur_config_replace(struct _xneur_config *p)
 	char *config_file_path_name		= get_file_path_name(NULL, CONFIG_NAME);
 	char *config_backup_file_path_name	= get_file_path_name(NULL, CONFIG_BCK_NAME);
 
-	log_message(LOG, "Moving config file from %s to %s", config_file_path_name, config_backup_file_path_name);
-	
+	log_message(LOG, _("Moving config file from %s to %s"), config_file_path_name, config_backup_file_path_name);
+
 	remove(config_backup_file_path_name);
 
 	if (rename(config_file_path_name, config_backup_file_path_name) != 0)
 	{
-		log_message(ERROR, "Can't move file!", config_backup_file_path_name);
+		log_message(ERROR, _("Can't move file!"), config_backup_file_path_name);
 
 		free(config_file_path_name);
 		free(config_backup_file_path_name);
@@ -747,8 +1000,8 @@ static void xneur_config_save_dicts(struct _xneur_config *p, int lang)
 {
 	if (!p->educate)
 		return;
-	
-	log_message(LOG, "Saving %s dictionary", p->get_lang_name(p, lang));
+
+	log_message(LOG, _("Saving %s dictionary"), p->get_lang_name(p, lang));
 
 	save_list(p->languages[lang].dicts, p->get_lang_dir(p, lang), DICT_NAME);
 }
@@ -788,7 +1041,7 @@ static void xneur_config_add_language(struct _xneur_config *p, const char *name,
 {
 	if (name == NULL || dir == NULL)
 	{
-		log_message(ERROR, "Can't add language with empty name or dir");
+		log_message(ERROR, _("Can't add language with empty name or dir"));
 		return;
 	}
 
@@ -798,7 +1051,7 @@ static void xneur_config_add_language(struct _xneur_config *p, const char *name,
 	p->languages[p->total_languages].name	= strdup(name);
 	p->languages[p->total_languages].dir	= strdup(dir);
 	p->languages[p->total_languages].group	= group;
-	
+
 	p->total_languages++;
 }
 
@@ -810,11 +1063,14 @@ static const char* xneur_config_get_log_level_name(struct _xneur_config *p)
 static void xneur_config_uninit(struct _xneur_config *p)
 {
 	free_structures(p);
-
+	
 	free(p->hotkeys);
 	free(p->sounds);
+	free(p->osds);
 	free(p->flags);
 	
+	free(p->actions);
+	
 	free(p);
 }
 
@@ -822,7 +1078,7 @@ struct _xneur_config* xneur_config_init(void)
 {
 	struct _xneur_config *p = (struct _xneur_config *) malloc(sizeof(struct _xneur_config));
 	bzero(p, sizeof(struct _xneur_config));
-		
+
 	if (!check_memory_attached(p))
 	{
 		free(p);
@@ -835,9 +1091,12 @@ struct _xneur_config* xneur_config_init(void)
 	p->sounds = (struct _xneur_file *) malloc(MAX_SOUNDS * sizeof(struct _xneur_file));
 	bzero(p->sounds, MAX_SOUNDS * sizeof(struct _xneur_file));
 
+	p->osds = (struct _xneur_file *) malloc(MAX_OSDS * sizeof(struct _xneur_file));
+	bzero(p->osds, MAX_OSDS * sizeof(struct _xneur_file));
+	
 	p->flags = (struct _xneur_file *) malloc(MAX_FLAGS * sizeof(struct _xneur_file));
 	bzero(p->flags, MAX_FLAGS * sizeof(struct _xneur_file));
-	
+
 	p->log_level			= LOG;
 	p->excluded_apps		= list_char_init();
 	p->auto_apps			= list_char_init();
@@ -845,11 +1104,17 @@ struct _xneur_config* xneur_config_init(void)
 	p->layout_remember_apps		= list_char_init();
 	p->window_layouts		= list_char_init();
 	p->draw_flag_apps		= list_char_init();
-		
+	p->abbreviations		= list_char_init();
+
+	p->actions = (struct _xneur_action *) malloc(sizeof(struct _xneur_action));
+	p->actions->action_hotkey = (struct _xneur_hotkey *) malloc(sizeof(struct _xneur_hotkey));
+	bzero(p->actions->action_hotkey, sizeof(struct _xneur_hotkey));
+	p->actions->action_command	= list_char_init();
+	
 	// Function mapping
 	p->get_home_dict_path		= get_home_file_path_name;
 	p->get_global_dict_path		= get_file_path_name;
-	
+
 	p->get_library_version		= xneur_config_get_library_version;
 	p->get_bool_name		= xneur_config_get_bool_name;
 
