@@ -34,8 +34,9 @@
 #define PIXMAPDIR			"pixmaps"
 
 #define MAX_FLAGS			4
-#define MAX_SOUNDS			14
-#define MAX_HOTKEYS			10
+#define MAX_SOUNDS			21
+#define MAX_OSDS			21
+#define MAX_HOTKEYS			11
 
 enum _flag_action
 {
@@ -47,7 +48,10 @@ enum _flag_action
 
 enum _sound_action
 {
-	SOUND_PRESS_KEY_LAYOUT_0 = 0,
+	SOUND_XNEUR_START = 0,
+	SOUND_XNEUR_RELOAD,
+	SOUND_XNEUR_STOP,
+	SOUND_PRESS_KEY_LAYOUT_0,
 	SOUND_PRESS_KEY_LAYOUT_1,
 	SOUND_PRESS_KEY_LAYOUT_2,
 	SOUND_PRESS_KEY_LAYOUT_3,
@@ -61,9 +65,39 @@ enum _sound_action
 	SOUND_CHANGE_SELECTED,
 	SOUND_TRANSLIT_SELECTED,
 	SOUND_CHANGECASE_SELECTED,
+	SOUND_REPLACE_ABBREVIATION,
+	SOUND_CORR_INCIDENTAL_CAPS,
+	SOUND_CORR_TWO_CAPITAL_LETTER,
+	SOUND_EXEC_USER_ACTION,
 	SOUND_NONE,
 };
 
+enum _osd_action
+{
+	OSD_XNEUR_START = 0,
+	OSD_XNEUR_RELOAD,
+	OSD_XNEUR_STOP,
+	OSD_PRESS_KEY_LAYOUT_0,
+	OSD_PRESS_KEY_LAYOUT_1,
+	OSD_PRESS_KEY_LAYOUT_2,
+	OSD_PRESS_KEY_LAYOUT_3,
+	OSD_ENABLE_LAYOUT_0,
+	OSD_ENABLE_LAYOUT_1,
+	OSD_ENABLE_LAYOUT_2,
+	OSD_ENABLE_LAYOUT_3,
+	OSD_AUTOMATIC_CHANGE_WORD,
+	OSD_MANUAL_CHANGE_WORD,
+	OSD_CHANGE_STRING,
+	OSD_CHANGE_SELECTED,
+	OSD_TRANSLIT_SELECTED,
+	OSD_CHANGECASE_SELECTED,
+	OSD_REPLACE_ABBREVIATION,
+	OSD_CORR_INCIDENTAL_CAPS,
+	OSD_CORR_TWO_CAPITAL_LETTER,
+	OSD_EXEC_USER_ACTION,
+	OSD_NONE,
+};
+
 enum _hotkey_action
 {
 	ACTION_CHANGE_WORD = 0,
@@ -76,23 +110,18 @@ enum _hotkey_action
 	ACTION_ENABLE_LAYOUT_1,
 	ACTION_ENABLE_LAYOUT_2,
 	ACTION_ENABLE_LAYOUT_3,
+	ACTION_REPLACE_ABBREVIATION,
 	ACTION_NONE,
 };
 
-enum _hotkey_modifier
+enum _change_action
 {
-	MODIFIER_NONE = 0,
-	MODIFIER_SHIFT,
-	MODIFIER_CONTROL,
-	MODIFIER_ALT,
-};
-
-enum _hotkey_key
-{
-	KEY_NONE = 0,
-	KEY_BREAK,
-	KEY_SCROLL_LOCK,
-	KEY_PRINT_SCREEN,
+	CORR_INCIDENTAL_CAPS = -2,
+	CORR_TWO_CAPITAL_LETTER,
+	ENABLE_LAYOUT_0,
+	ENABLE_LAYOUT_1,
+	ENABLE_LAYOUT_2,
+	ENABLE_LAYOUT_3,
 };
 
 struct _xneur_language
@@ -129,6 +158,12 @@ struct _xneur_data
 	int manual_mode;
 };
 
+struct _xneur_action
+{
+	struct _xneur_hotkey *action_hotkey;
+	struct _list_char *action_command;
+};
+
 struct _xneur_config
 {
 	char *version;
@@ -145,7 +180,9 @@ struct _xneur_config
 	struct _xneur_language *languages;		// Array of languages used in program
 	struct _xneur_hotkey *hotkeys;			// Array of hotkeys used in program
 	struct _xneur_file *sounds;			// Array of sounds for actions
+	struct _xneur_file *osds;			// Array of OSDs for actions
 
+	int   manual_mode;				// Enable manual processing mode
 	int   log_level;				// Maximum level of log messages to print
 	int   send_delay;				// Delay before send event (in milliseconds)
 	int   total_languages;				// Total languages to work with
@@ -158,10 +195,23 @@ struct _xneur_config
 	int   remember_layout;				// Remember layout for each of window
 	int   save_selection;				// Save selection after convert
 	int   save_keyboard_log;			// Save keyboard log
+	int   correct_incidental_caps;		// Change iNCIDENTAL CapsLock
+	int   correct_two_capital_letter;	// Change two CApital letter
+	int   flush_buffer_when_press_enter;  // Flush internal buffer when pressed Enter
+	int   dont_process_when_press_enter;   // Don't correct word when pressed Enter
 
+	int   show_osd;					// Show OSD 
+	char  *osd_font;
+
+	int   abbr_ignore_layout;			// Ignore keyboard layout for abbreviations
+	
 	struct _list_char *draw_flag_apps;
 	struct _xneur_file *flags;			// Array of flag pixmaps for draw
 
+	struct _list_char *abbreviations;		// Array of words to replace
+
+	struct _xneur_action *actions;
+
 	char* (*get_home_dict_path) (const char *dir_name, const char *file_name);
 	char* (*get_global_dict_path) (const char *dir_name, const char *file_name);
 	const char* (*get_bool_name) (int option);
