@@ -7,20 +7,30 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  (c) XNeur Team 2006
+ *  Copyright (C) 2006-2008 XNeur Team
  *
  */
 
-#include <pcre.h>
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
+
+#ifdef WITH_PCRE
+#  include <pcre.h>
+#endif
+
 #include <string.h>
 
 #include "types.h"
+#include "log.h"
+
+#ifdef WITH_PCRE
 
 int check_regexp_match(const char *str, const char *pattern)
 {
@@ -28,19 +38,32 @@ int check_regexp_match(const char *str, const char *pattern)
 	const char *error;
 	int erroffset;
 
-	pcre *re = pcre_compile((char *)pattern, options, &error, &erroffset, NULL);
+	pcre *re = pcre_compile((char *) pattern, options, &error, &erroffset, NULL);
 	if (!re)
 		return FALSE;
-	
+
+	int str_len = strlen(str);
+
 	int ovector[2];
-	int count = pcre_exec(re, NULL, (char *)str, strlen(str), 0, 0, ovector, 2);
+	int count = pcre_exec(re, NULL, (char *) str, str_len, 0, 0, ovector, 2);
 	pcre_free(re);
-	
+
 	if (count <= 0)
 		return FALSE;
-	
-	if (ovector[0] != 0 || ovector[1] != strlen(str))
+
+	if (ovector[0] != 0 || ovector[1] != str_len)
 		return FALSE;
 
-	return TRUE;			
+	return TRUE;
 }
+
+#else
+
+int check_regexp_match(const char *str, const char *pattern)
+{
+	str = str;
+	pattern = pattern;
+	return FALSE;
+}
+
+#endif
