@@ -7,16 +7,18 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  (c) XNeur Team 2006
+ *  Copyright (C) 2006-2008 XNeur Team
  *
  */
 
+#include "debug.h"
+
 #ifndef _TYPES_H_
 #define _TYPES_H_
 
@@ -25,9 +27,4 @@
 
 #define NULLSYM		0
 
-#define ERROR		0
-#define WARNING		1
-#define LOG		2
-#define DEBUG		3
-
 #endif /* _TYPES_H_ */
