@@ -24,7 +24,7 @@
 #define EVENT_PRESS_MASK	KeyPressMask 
 #define EVENT_RELEASE_MASK	KeyReleaseMask 
 #define POINTER_MOTION_MASK	PointerMotionMask
-#define FOCUS_CHANGE_MASK	LeaveWindowMask | EnterWindowMask | FocusChangeMask
+#define FOCUS_CHANGE_MASK	FocusChangeMask | LeaveWindowMask | EnterWindowMask
 #define BUTTON_HANDLE_MASK	ButtonPressMask
 
 static const int groups[4] = {0x00000000, 0x00002000, 0x00004000, 0x00006000};
