@@ -29,6 +29,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <signal.h>
+#include <pthread.h>
 
 #include "xnconfig.h"
 #include "xnconfig_files.h"
@@ -53,6 +54,7 @@
 #include "conversion.h"
 #include "sound.h"
 
+#include "xOSD.h"
 #include "xprogram.h"
 
 #define KLB_NO_ACTION           0	// Modifier, function etc
@@ -78,7 +80,7 @@ static int get_auto_action(KeySym key, int modifier_mask)
 	// Null symbol
 	if (key == 0)
 		return KLB_NO_ACTION;
-	
+
 	// Cursor keys
 	if (IsCursorKey(key))
 		return KLB_CLEAR;
@@ -131,8 +133,8 @@ static int get_auto_action(KeySym key, int modifier_mask)
 		case XK_Delete:
 			return KLB_NO_ACTION;
 		case XK_Return:
-			return KLB_ENTER;
 		case XK_Tab:
+			return KLB_ENTER;
 		case XK_space:
 		case XK_equal:
 		case XK_plus:
@@ -143,39 +145,45 @@ static int get_auto_action(KeySym key, int modifier_mask)
 		case XK_question:
 			return KLB_SPACE;
 	}
-	
+
 	if (modifier_mask & ControlMask || modifier_mask & Mod1Mask || modifier_mask & Mod4Mask)
 		return KLB_NO_ACTION;
 
-	int sound;
-	int lang = get_cur_lang(); 
+	int sound = SOUND_NONE;
+	int osd = OSD_NONE;
+	int lang = get_cur_lang();
 	switch (lang)
 	{
 		default:
 		case 0:
 		{
 			sound = SOUND_PRESS_KEY_LAYOUT_0;
+			osd = OSD_PRESS_KEY_LAYOUT_0;
 			break;
 		}
 		case 1:
 		{
 			sound = SOUND_PRESS_KEY_LAYOUT_1;
+			osd = OSD_PRESS_KEY_LAYOUT_1;
 			break;
 		}
 		case 2:
 		{
 			sound = SOUND_PRESS_KEY_LAYOUT_2;
+			osd = OSD_PRESS_KEY_LAYOUT_2;
 			break;
 		}
 		case 3:
 		{
-			sound = SOUND_PRESS_KEY_LAYOUT_0;
+			sound = SOUND_PRESS_KEY_LAYOUT_3;
+			osd = OSD_PRESS_KEY_LAYOUT_3;
 			break;
 		}
 	}
 
 	play_file(sound);
-
+	osd_show(xconfig->osds[osd].file);
+	
 	return KLB_ADD_SYM;
 }
 
@@ -183,17 +191,17 @@ static void xprogram_cursor_update(struct _xprogram *p)
 {
 	if (!p->focus->draw_flag(p->focus, p->event->event.xmotion.window))
 	{
-		p->cursor->hide_flag(p->cursor);
+		p->cursor->hide_flag();
 		return;
 	}
 
 	int root_x, root_y, win_x, win_y;
 	Window root_window, child_window;
 	unsigned int dummyU;
-					
-	XQueryPointer(main_window->display, p->focus->owner_window, &root_window, &child_window, &root_x, &root_y, &win_x, &win_y, &dummyU);			
 
-	p->cursor->show_flag(p->cursor, root_x + 10, root_y + 10);
+	XQueryPointer(main_window->display, p->focus->owner_window, &root_window, &child_window, &root_x, &root_y, &win_x, &win_y, &dummyU);
+
+	p->cursor->show_flag(root_x + 10, root_y + 10);
 }
 
 static void xprogram_layout_update(struct _xprogram *p)
@@ -217,7 +225,7 @@ static void xprogram_layout_update(struct _xprogram *p)
 		sprintf(text_to_find, "%s", last_app_name);
 	else
 		sprintf(text_to_find, "%d", (int) p->last_window);
-	
+
 	if (last_app_name != NULL)
 		free(last_app_name);
 
@@ -225,7 +233,7 @@ static void xprogram_layout_update(struct _xprogram *p)
 	for (int lang = 0; lang < xconfig->total_languages; lang++)
 	{
 		sprintf(window_layouts, "%s %d", text_to_find, lang);
-		
+
 		if (!xconfig->window_layouts->exist(xconfig->window_layouts, window_layouts, BY_PLAIN))
 			continue;
 
@@ -235,7 +243,7 @@ static void xprogram_layout_update(struct _xprogram *p)
 	// Save layout for old window
 	sprintf(window_layouts, "%s %d", text_to_find, p->last_layout);
 	xconfig->window_layouts->add(xconfig->window_layouts, window_layouts);
-	
+
 	struct _list_char_data* curr_app = NULL;
 
 	char *curr_app_name = get_wm_class_name(p->focus->owner_window);
@@ -262,14 +270,14 @@ static void xprogram_layout_update(struct _xprogram *p)
 		free(window_layouts);
 
 		switch_lang(lang);
-		log_message(DEBUG, "Restore layout group to %d", xconfig->get_lang_group(xconfig, lang));
+		log_message(DEBUG, _("Restore layout group to %d"), xconfig->get_lang_group(xconfig, lang));
 		return;
 	}
 
 	free(text_to_find);
 	free(window_layouts);
 
-	log_message(DEBUG, "Store default layout group to %d", xconfig->default_group);
+	log_message(DEBUG, _("Store default layout group to %d"), xconfig->default_group);
 	switch_group(xconfig->default_group);
 }
 
@@ -278,13 +286,14 @@ static void xprogram_update(struct _xprogram *p)
 	p->last_window = p->focus->owner_window;
 
 	int status = p->focus->get_focus_status(p->focus, &p->app_forced_mode, &p->app_focus_mode);
+	p->event->set_owner_window(p->event, p->focus->owner_window);
+	
+	if (status == FOCUS_UNCHANGED)
+		return;
 
-	if (status != FOCUS_UNCHANGED)
-	{
-		p->layout_update(p);
-		p->string->save_and_clear(p->string, p->last_window);
-	}
-
+	p->layout_update(p);
+	p->string->save_and_clear(p->string, p->last_window);
+		
 	if (status == FOCUS_NONE)
 		return;
 
@@ -292,7 +301,6 @@ static void xprogram_update(struct _xprogram *p)
 	if (p->app_focus_mode == FOCUS_EXCLUDED)
 		listen_mode = LISTEN_DONTGRAB_INPUT;
 
-	p->event->set_owner_window(p->event, p->focus->owner_window);
 	p->focus->update_events(p->focus, listen_mode);
 }
 
@@ -303,7 +311,7 @@ static void xprogram_process_input(struct _xprogram *p)
 	while (1)
 	{
 		int type = p->event->get_next_event(p->event);
-		
+
 		if (p->event->event.xany.window == main_window->flag_window)
 			continue;
 
@@ -315,26 +323,45 @@ static void xprogram_process_input(struct _xprogram *p)
 				XClientMessageEvent *cme = (XClientMessageEvent *) &(p->event->event);
 				if (cme->message_type == main_window->close_atom)
 				{
-					log_message(LOG, "Exitting from main cycle");
+					log_message(LOG, _("Exitting from main cycle"));
 					return;
 				}
+				break;
 			}
 			case KeyPress:
 			{
-				log_message(TRACE, "Received KeyPress (event type %d)", type);
+				log_message(TRACE, _("Received KeyPress (event type %d)"), type);
+				// Save received event
+				p->event->default_event = p->event->event;
+				// Processing received event 
 				p->on_key_action(p);
+				// Restore event
+				if (p->event->default_event.xkey.keycode != 0)
+				{
+					p->event->event = p->event->default_event;
+					p->event->send_next_event(p->event);		
+				}
 				break;
 			}
 			case KeyRelease:
 			{
-				log_message(TRACE, "Received KeyRelease (event type %d)", type);
+				log_message(TRACE, _("Received KeyRelease (event type %d)"), type);
+				// Resend special key back to window
+				if (p->event->default_event.xkey.keycode != 0)
+					p->event->send_next_event(p->event);
 				break;
 			}
 			case FocusIn:
-				log_message(TRACE, "Received FocusIn (event type %d)", type);
 			case LeaveNotify:
 			case EnterNotify:
 			{
+				if (type == FocusIn)
+					log_message(TRACE, _("Received FocusIn (event type %d)"), type);
+				else if (type == LeaveNotify)
+					log_message(TRACE, _("Received LeaveNotify (event type %d)"), type);
+				else if (type == EnterNotify)
+					log_message(TRACE, _("Received EnterNotify (event type %d)"), type);
+
 				p->last_layout = get_active_keyboard_group();
 
 				p->update(p);
@@ -343,7 +370,7 @@ static void xprogram_process_input(struct _xprogram *p)
 			}
 			case FocusOut:
 			{
-				log_message(TRACE, "Received FocusOut (event type %d)", type);
+				log_message(TRACE, _("Received FocusOut (event type %d)"), type);
 
 				p->last_layout = get_active_keyboard_group();
 				p->update(p);
@@ -351,45 +378,49 @@ static void xprogram_process_input(struct _xprogram *p)
 			}
 			case SelectionNotify:
 			{
-				p->process_selection(p);
+				log_message(TRACE, _("Received SelectionNotify (event type %d)"), type);
+				p->process_selection_notify(p);
+				break;
+			}
+			case SelectionRequest:
+			{
+				log_message(TRACE, _("Received SelectionRequest (event type %d)"), type);
+				p->process_selection_notify(p);
 				break;
 			}
 			case ButtonPress:
 			{
-				// Falling down
 				p->string->save_and_clear(p->string, p->focus->owner_window);
-				log_message(TRACE, "Received ButtonPress on window %d  (event type %d)", p->event->event.xbutton.window, type);
-								
+				log_message(TRACE, _("Received ButtonPress on window %d  (event type %d)"), p->event->event.xbutton.window, type);
+
 				// Unfreeze and resend grabbed event
 				XAllowEvents(main_window->display, ReplayPointer, CurrentTime);
-
 				break;
 			}
 			case MotionNotify:
 			{
-				//log_message(TRACE, "Received Motion Notify");
-				p->cursor_update(p);	
-				break;				
+				//log_message(TRACE, _("Received Motion Notify on %d"), p->event->event.xmotion.window);
+				p->cursor_update(p);
+				break;
 			}
 			case PropertyNotify:
 			{
-				
 				if (XInternAtom(main_window->display, "XKLAVIER_STATE", FALSE) == p->event->event.xproperty.atom)
 				{
-					log_message(TRACE, "Received Property Notify (layout switch event) (event type %d)", type);
-					
+					log_message(TRACE, _("Received Property Notify (layout switch event) (event type %d)"), type);
+
 					// Flush string
 					//p->string->clear(p->string);
-					
+
 					// Update flag
 					p->cursor_update(p);
 				}
-			
+
 				break;
 			}
 			default:
 			{
-				log_message(DEBUG, "Uncatched event with type %d)", type);
+				log_message(DEBUG, _("Uncatched event with type %d)"), type);
 				break;
 			}
 		}
@@ -398,27 +429,54 @@ static void xprogram_process_input(struct _xprogram *p)
 
 static void xprogram_change_lang(struct _xprogram *p, int new_lang)
 {
-	log_message(DEBUG, "Changing language from %s to %s", xconfig->get_lang_name(xconfig, get_cur_lang()), xconfig->get_lang_name(xconfig, new_lang));
-	p->string->set_key_code(p->string, new_lang);
+	log_message(DEBUG, _("Changing language from %s to %s"), xconfig->get_lang_name(xconfig, get_cur_lang()), xconfig->get_lang_name(xconfig, new_lang));
+	p->string->set_lang_mask(p->string, new_lang);
 	switch_lang(new_lang);
 }
 
-static void xprogram_process_selection(struct _xprogram *p)
+static void xprogram_change_incidental_caps(struct _xprogram *p)
+{
+	log_message(DEBUG, _("Correcting iNCIDENTAL CapsLock"));
+	// Change modifier mask
+	p->string->set_uncaps_mask(p->string);
+	// Change CAPS if need
+	if (get_key_state(XK_Caps_Lock))
+	{
+		int xkb_opcode, xkb_event, xkb_error;
+		int xkb_lmaj = XkbMajorVersion;
+		int xkb_lmin = XkbMinorVersion;
+		if (XkbLibraryVersion(&xkb_lmaj, &xkb_lmin) && XkbQueryExtension(main_window->display, &xkb_opcode, &xkb_event, &xkb_error,
+			       &xkb_lmaj, &xkb_lmin))
+			{
+				XkbLockModifiers (main_window->display, XkbUseCoreKbd, LockMask, 0);
+			}
+	}
+}
+
+static void xprogram_change_two_capital_letter(struct _xprogram *p)
+{
+	log_message(DEBUG, _("Correcting two CApital letter"));
+	// Change modifier mask
+	p->string->keycode_modifiers[1] = p->string->keycode_modifiers[1] & (~ShiftMask);
+}
+
+static void xprogram_process_selection_notify(struct _xprogram *p)
 {
 	char *event_text = get_selected_text(&p->event->event.xselection);
 	if (event_text == NULL)
+	{
+		p->selected_mode = ACTION_NONE;
 		return;
+	}
 
 	char *selected_text = strdup(event_text);
 	XFree(event_text);
 
-	if (p->selected_mode == ACTION_CHANGE_SELECTED || p->selected_mode == ACTION_CHANGECASE_SELECTED)
-		main_window->xkeymap->convert_text_to_ascii(main_window->xkeymap, selected_text);
-
 	if (p->selected_mode == ACTION_TRANSLIT_SELECTED)
 		convert_text_to_translit(&selected_text);
 
 	p->string->set_content(p->string, selected_text);
+	free(selected_text);
 
 	if (p->selected_mode == ACTION_CHANGECASE_SELECTED)
 	{
@@ -427,20 +485,28 @@ static void xprogram_process_selection(struct _xprogram *p)
 	}
 	else
 		p->change_lang(p, get_next_lang(get_cur_lang()));
-	
-	if (p->selected_mode == ACTION_CHANGECASE_SELECTED)
-		play_file(SOUND_CHANGECASE_SELECTED);
-	else if (p->selected_mode == ACTION_TRANSLIT_SELECTED)
-		play_file(SOUND_TRANSLIT_SELECTED);
-	else if (p->selected_mode == ACTION_CHANGE_SELECTED)
+
+	if (p->selected_mode == ACTION_CHANGE_SELECTED)
+	{
 		play_file(SOUND_CHANGE_SELECTED);
-	
+		osd_show(xconfig->osds[OSD_CHANGE_SELECTED].file);
+	}
+	else if (p->selected_mode == ACTION_CHANGECASE_SELECTED)
+	{
+		play_file(SOUND_CHANGECASE_SELECTED);
+		osd_show(xconfig->osds[OSD_CHANGECASE_SELECTED].file);
+	}
+	else if (p->selected_mode == ACTION_TRANSLIT_SELECTED)
+	{
+		play_file(SOUND_TRANSLIT_SELECTED);
+		osd_show(xconfig->osds[OSD_TRANSLIT_SELECTED].file);
+	}
 	p->focus->update_events(p->focus, LISTEN_DONTGRAB_INPUT);	// Disable receiving events
-	
-	// Block events of keyboard 
+
+	// Block events of keyboard
 	set_event_mask(p->focus->owner_window, None);
 	grab_spec_keys(p->focus->owner_window, FALSE);
-	
+
 	p->send_string_silent(p, FALSE);
 
 	on_selection_converted();
@@ -449,20 +515,24 @@ static void xprogram_process_selection(struct _xprogram *p)
 		p->event->send_selection(p->event, p->string->cur_pos);
 
 	p->update(p);
-	
+
 	p->string->save_and_clear(p->string, p->focus->owner_window);
-	free(selected_text);
+	p->selected_mode = ACTION_NONE;
 }
 
 static void xprogram_on_key_action(struct _xprogram *p)
 {
 	KeySym key = p->event->get_cur_keysym(p->event);
 
+	// Delete language modifier mask
 	int modifier_mask = p->event->get_cur_modifiers(p->event);
-	if (p->modifier_mask != NO_MODIFIER_MASK) 
+	
+	int user_action = get_user_action(key, modifier_mask);
+	if (user_action >= 0) 
 	{
-		p->event->event.xkey.state = p->modifier_mask;
-		p->modifier_mask = NO_MODIFIER_MASK;
+		p->perform_user_action(p, user_action);
+	    p->event->default_event.xkey.keycode = 0;
+		return;
 	}
 	
 	enum _hotkey_action manual_action = get_manual_action(key, modifier_mask);
@@ -473,6 +543,25 @@ static void xprogram_on_key_action(struct _xprogram *p)
 	p->perform_auto_action(p, auto_action);
 }
 
+static void xprogram_perform_user_action(struct _xprogram *p, int action)
+{
+	if (p) {};
+	
+	log_message(DEBUG, _("Execute user action \"%s\""), xconfig->actions->action_command->data[action].string); 
+	
+	pthread_attr_t action_thread_attr;
+	pthread_attr_init(&action_thread_attr);
+	pthread_attr_setdetachstate(&action_thread_attr, PTHREAD_CREATE_DETACHED);
+
+	pthread_t action_thread;
+	pthread_create(&action_thread, &action_thread_attr,(void *) &system, (void *) xconfig->actions->action_command->data[action].string);
+	
+	pthread_attr_destroy(&action_thread_attr);
+	
+	play_file(SOUND_EXEC_USER_ACTION);
+	osd_show(xconfig->osds[OSD_EXEC_USER_ACTION].file, xconfig->actions->action_command->data[action].string);
+}
+
 static void xprogram_perform_auto_action(struct _xprogram *p, int action)
 {
 	struct _xstring *string = p->string;
@@ -493,59 +582,65 @@ static void xprogram_perform_auto_action(struct _xprogram *p, int action)
 		case KLB_SPACE:
 		case KLB_ADD_SYM:
 		{
+			if ((action == KLB_ENTER) && (xconfig->dont_process_when_press_enter))
+			{
+				action = KLB_ADD_SYM;
+			}
+			
 			if (p->changed_manual == MANUAL_FLAG_SET)
 				p->changed_manual = MANUAL_FLAG_NEED_FLUSH;
 
 			p->last_action = action;
 
-			// Save event
-			XEvent tmp = p->event->event;
 			char sym = main_window->xkeymap->get_cur_ascii_char(main_window->xkeymap, p->event->event);
-			
+
 			if (action == KLB_ADD_SYM)
 			{
 				if (p->changed_manual == MANUAL_FLAG_NEED_FLUSH)
 					p->changed_manual = MANUAL_FLAG_UNSET;
 
 				// Add symbol to internal bufer
-				p->string->add_symbol(p->string, sym, p->event->event.xkey.keycode, p->event->event.xkey.state);
+				int modifier_mask = groups[get_cur_lang()] | p->event->get_cur_modifiers(p->event);
+				p->string->add_symbol(p->string, sym, p->event->event.xkey.keycode, modifier_mask);
+
 				return;
 			}
-	
+
 			// Block events of keyboard (push to event queue)
 			set_event_mask(p->focus->owner_window, None);
-			grab_spec_keys(p->focus->owner_window, FALSE);
-			grab_keyboard(p->focus->owner_window, TRUE); 
+
+			// Check two capital letter
+			if (xconfig->correct_two_capital_letter)
+				p->check_tcl_last_word(p);
+			
+			// Check incidental caps
+			if (xconfig->correct_incidental_caps)
+				p->check_caps_last_word(p); 
 			
 			// Checking word
-			if (p->changed_manual == MANUAL_FLAG_UNSET)	
-				p->check_last_word(p);
-			
-			// Restore event
-			p->event->event = tmp;
+			if (p->changed_manual == MANUAL_FLAG_UNSET)
+				p->check_lang_last_word(p); 
 
 			// Add symbol to internal bufer
+			p->event->event = p->event->default_event;
 			int modifier_mask = groups[get_cur_lang()] | p->event->get_cur_modifiers(p->event);
 			p->string->add_symbol(p->string, sym, p->event->event.xkey.keycode, modifier_mask);
 
-			// Resend special key back to window
+			// Send Event
 			p->event->send_next_event(p->event);
+			p->event->default_event.xkey.keycode = 0;
+						
+			// Unblock keyboard
+		
+			set_event_mask(p->focus->owner_window, POINTER_MOTION_MASK | INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK);
 
-			// Resend blocked events back to window (from the event queue)
-			while (XEventsQueued(main_window->display, QueuedAlready)) 
+			p->changed_manual = MANUAL_FLAG_NEED_FLUSH;
+			
+			if ((action == KLB_ENTER) && (xconfig->flush_buffer_when_press_enter))
 			{
-				p->event->get_next_event(p->event);
-				if (p->event->event.xany.window == main_window->flag_window)
-					continue;
-				p->event->send_next_event(p->event);
+				p->string->save_and_clear(p->string, p->focus->owner_window);
 			}
 			
-			// Unblock keyboard 
-			grab_keyboard(p->focus->owner_window, FALSE);					
-			set_event_mask(p->focus->owner_window, POINTER_MOTION_MASK | INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_PRESS_MASK);
-			grab_spec_keys(p->focus->owner_window, TRUE);
-			
-			p->changed_manual = MANUAL_FLAG_NEED_FLUSH;
 			return;
 		}
 	}
@@ -561,7 +656,8 @@ static int xprogram_perform_manual_action(struct _xprogram *p, enum _hotkey_acti
 		{
 			xconfig->set_manual_mode(xconfig, !xconfig->is_manual_mode(xconfig));
 
-			log_message(DEBUG, "Manual mode changed to %s", xconfig->get_bool_name(xconfig->is_manual_mode(xconfig)));
+			log_message(DEBUG, _("Manual mode changed to %s"), xconfig->get_bool_name(xconfig->is_manual_mode(xconfig)));
+			p->event->default_event.xkey.keycode = 0;
 			return TRUE;
 		}
 		case ACTION_CHANGE_SELECTED:
@@ -569,18 +665,20 @@ static int xprogram_perform_manual_action(struct _xprogram *p, enum _hotkey_acti
 		case ACTION_CHANGECASE_SELECTED:
 		{
 			p->selected_mode = action;
-			do_selection_request();
+			do_selection_notify();
+			p->event->default_event.xkey.keycode = 0;
 			return TRUE;
 		}
 		case ACTION_CHANGE_STRING:	// User needs to change current string
 		{
 			p->change_lang(p, get_next_lang(get_cur_lang()));
-			
+
 			p->focus->update_events(p->focus, LISTEN_DONTGRAB_INPUT);	// Disable receiving events
 			p->send_string_silent(p, TRUE);
-			p->update(p);			
-			
+			p->update(p);
+
 			play_file(SOUND_CHANGE_STRING);
+			osd_show(xconfig->osds[OSD_CHANGE_STRING].file);
 			p->cursor_update(p);
 			break;
 		}
@@ -598,20 +696,104 @@ static int xprogram_perform_manual_action(struct _xprogram *p, enum _hotkey_acti
 
 			set_event_mask(p->focus->owner_window, POINTER_MOTION_MASK | INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_PRESS_MASK);
 			grab_spec_keys(p->focus->owner_window, TRUE);
-			
+
 			play_file(SOUND_MANUAL_CHANGE_WORD);
+			osd_show(xconfig->osds[OSD_MANUAL_CHANGE_WORD].file);
 			p->cursor_update(p);
+			p->event->default_event.xkey.keycode = 0;
+			
 			break;
 		}
 		case ACTION_ENABLE_LAYOUT_0:
+		{
+			switch_group(0);
+			p->event->default_event.xkey.keycode = 0;
+			play_file(SOUND_ENABLE_LAYOUT_0);
+			osd_show(xconfig->osds[OSD_ENABLE_LAYOUT_0].file);
+			break;
+		}
 		case ACTION_ENABLE_LAYOUT_1:
+		{
+			switch_group(1);
+			p->event->default_event.xkey.keycode = 0;
+			play_file(SOUND_ENABLE_LAYOUT_1);
+			osd_show(xconfig->osds[OSD_ENABLE_LAYOUT_1].file);
+			break;
+		}
 		case ACTION_ENABLE_LAYOUT_2:
+		{
+			switch_group(2);
+			p->event->default_event.xkey.keycode = 0;
+			play_file(SOUND_ENABLE_LAYOUT_2);
+			osd_show(xconfig->osds[OSD_ENABLE_LAYOUT_2].file);
+			break;
+		}
 		case ACTION_ENABLE_LAYOUT_3:
 		{
-			switch_group(action - ACTION_ENABLE_LAYOUT_0);
-			play_file(action);
+			switch_group(3);
+			p->event->default_event.xkey.keycode = 0;
+			play_file(SOUND_ENABLE_LAYOUT_3);
+			osd_show(xconfig->osds[OSD_ENABLE_LAYOUT_3].file);
 			break;
 		}
+		case ACTION_REPLACE_ABBREVIATION: // User needs to replace acronym
+		{
+			// Check last word to acronym list
+			// FIX_ME!!! Possible memory leak
+			char *word = get_last_word(p->string->get_utf_string(p->string));
+			
+			if (!word)
+				return FALSE;
+
+			for (int words = 0; words < xconfig->abbreviations->data_count; words++)
+			{
+				char *string		= strdup(xconfig->abbreviations->data[words].string);
+				char *replacement	= strsep(&string, " ");
+
+				if (xconfig->abbr_ignore_layout)
+				{
+					KeyCode *dummy_kc = malloc(strlen(replacement) * sizeof(KeyCode));
+					int *dummy_kc_mod = malloc(strlen(replacement) * sizeof(int));
+					main_window->xkeymap->convert_text_to_ascii(main_window->xkeymap, replacement, dummy_kc, dummy_kc_mod);
+					
+					dummy_kc = realloc(dummy_kc, strlen(word) * sizeof(KeyCode));
+					dummy_kc_mod = realloc(dummy_kc_mod, strlen(word) * sizeof(int));
+					main_window->xkeymap->convert_text_to_ascii(main_window->xkeymap, word, dummy_kc, dummy_kc_mod);
+					
+					free(dummy_kc);
+					free(dummy_kc_mod);
+				}
+				
+				if (strcmp(replacement, word) != 0)
+				{
+					free(replacement);
+					continue;
+				}
+				// Replace Abbreviation
+				log_message (DEBUG, _("Found Abbreviation '%s' '%s'. Replacing to '%s'."), replacement, word, string);
+				
+				set_event_mask(p->focus->owner_window, None);
+				grab_spec_keys(p->focus->owner_window, FALSE);				
+
+				p->event->send_backspaces(p->event, strlen(get_last_word(p->string->content)));
+				p->string->set_content(p->string, string);
+				p->send_string_silent(p, FALSE);
+				
+				set_event_mask(p->focus->owner_window, POINTER_MOTION_MASK | INPUT_HANDLE_MASK | FOCUS_CHANGE_MASK | EVENT_PRESS_MASK);
+				grab_spec_keys(p->focus->owner_window, TRUE);
+
+				play_file(SOUND_REPLACE_ABBREVIATION);
+				osd_show(xconfig->osds[OSD_REPLACE_ABBREVIATION].file);
+				p->string->save_and_clear(p->string, p->focus->owner_window);
+
+				free(replacement);
+				
+				p->event->default_event.xkey.keycode = 0;
+				
+				return TRUE;
+			}
+			return FALSE;
+		}
 	}
 
 	// When CHANGE_STRING or CHANGE_WORD actions occured
@@ -620,7 +802,7 @@ static int xprogram_perform_manual_action(struct _xprogram *p, enum _hotkey_acti
 	return TRUE;
 }
 
-static void xprogram_check_last_word(struct _xprogram *p)
+static void xprogram_check_lang_last_word(struct _xprogram *p)
 {
 	if (p->app_forced_mode == FORCE_MODE_MANUAL)
 		return;
@@ -637,7 +819,7 @@ static void xprogram_check_last_word(struct _xprogram *p)
 	int new_lang = get_word_lang(word, cur_lang);
 	if (new_lang == NO_LANGUAGE)
 	{
-		log_message(DEBUG, "No language found to change to");
+		log_message(DEBUG, _("No language found to change to"));
 		return;
 	}
 
@@ -646,40 +828,110 @@ static void xprogram_check_last_word(struct _xprogram *p)
 
 	p->change_word(p, new_lang);
 	play_file(SOUND_AUTOMATIC_CHANGE_WORD);
+	osd_show(xconfig->osds[OSD_AUTOMATIC_CHANGE_WORD].file);
 	p->cursor_update(p);
 }
 
+static void xprogram_check_caps_last_word(struct _xprogram *p)
+{
+	int offset = get_last_word_offset(p->string->content, p->string->cur_pos);
+	
+	if ((p->string->keycode_modifiers[offset] & LockMask) && (p->string->keycode_modifiers[offset] & ShiftMask))
+	{
+		for (int i = 1; i < p->string->cur_pos - offset; i++)
+		{
+			if ((p->string->keycode_modifiers[offset+i] & LockMask) && (p->string->keycode_modifiers[offset+i] & ShiftMask))
+				return;
+			if (!(p->string->keycode_modifiers[offset+i] & LockMask))
+				return;
+		}
+		
+		p->change_word(p, CORR_INCIDENTAL_CAPS);
+		play_file(SOUND_CORR_INCIDENTAL_CAPS);
+		osd_show(xconfig->osds[OSD_CORR_INCIDENTAL_CAPS].file);
+		return;
+	}
+	
+	return;
+}
+
+static void xprogram_check_tcl_last_word(struct _xprogram *p)
+{
+	int offset = get_last_word_offset(p->string->content, p->string->cur_pos);
+
+	if (p->string->cur_pos - offset <= 2)
+		return;
+	else
+		if (p->string->content[offset+2] == ' ')
+			return;
+
+	if ((p->string->keycode_modifiers[offset] & ShiftMask) &&
+		(p->string->keycode_modifiers[offset+1] & ShiftMask))
+	{
+		for (int i = 2; i < p->string->cur_pos - offset; i++)
+			if (p->string->keycode_modifiers[offset+i] & ShiftMask)
+				return;
+		
+		p->change_word(p, CORR_TWO_CAPITAL_LETTER);
+		play_file(SOUND_CORR_TWO_CAPITAL_LETTER);
+		osd_show(xconfig->osds[OSD_CORR_TWO_CAPITAL_LETTER].file);
+		return;
+	}
+	
+	return;
+}
+
 static void xprogram_send_string_silent(struct _xprogram *p, int send_backspaces)
 {
 	if (p->string->cur_pos == 0)
 	{
-		log_message(DEBUG, "No string to change");
+		log_message(DEBUG, _("No string to change"));
 		return;
 	}
 
-	log_message(DEBUG, "Processing string '%s'", p->string->content);
+	log_message(DEBUG, _("Processing string '%s'"), p->string->content);
 
 	int bcount = p->string->cur_pos;
 	if (send_backspaces == FALSE)
 		bcount = 0;
 
 	p->event->send_backspaces(p->event, bcount);		// Delete old string
-	p->event->send_string(p->event, p->string);			// Send new string
+	p->event->send_string(p->event, p->string);		// Send new string
 }
 
-static void xprogram_change_word(struct _xprogram *p, int new_lang)
+static void xprogram_change_word(struct _xprogram *p, enum _change_action action)
 {
 	int offset = get_last_word_offset(p->string->content, p->string->cur_pos);
-	
+
 	// Shift fields to point to begin of word
 	p->string->content		+= offset;
 	p->string->keycode		+= offset;
 	p->string->keycode_modifiers	+= offset;
 	p->string->cur_pos		-= offset;
 
-	p->change_lang(p, new_lang);
+	switch (action)
+	{
+		case CORR_INCIDENTAL_CAPS:
+		{
+			p->change_incidental_caps(p);
+			break;
+		}
+		case CORR_TWO_CAPITAL_LETTER:
+		{
+			p->change_two_capital_letter(p);
+			break;
+		}
+		case ENABLE_LAYOUT_0:
+		case ENABLE_LAYOUT_1:
+		case ENABLE_LAYOUT_2:
+		case ENABLE_LAYOUT_3:
+		{
+			p->change_lang(p, action);
+			break;
+		}
+	}
 	p->send_string_silent(p, TRUE);
-	
+
 	// Revert fields back
 	p->string->content		-= offset;
 	p->string->keycode		-= offset;
@@ -692,7 +944,7 @@ static void xprogram_add_word_to_dict(struct _xprogram *p, int new_lang)
 	char *word = get_last_word(p->string->content);
 	if (word == NULL)
 		return;
-	
+
 	int len = strlen(word);
 	char *low_word = lower_word(word, len);
 
@@ -720,17 +972,17 @@ static void xprogram_add_word_to_dict(struct _xprogram *p, int new_lang)
 	struct _list_char *curr_dicts = xconfig->languages[curr_lang].dicts;
 	if (curr_dicts->exist(curr_dicts, low_word, BY_PLAIN))
 	{
-		log_message(DEBUG, "Remove word '%s' from %s dictionary", low_word, xconfig->get_lang_name(xconfig, curr_lang));
+		log_message(DEBUG, _("Remove word '%s' from %s dictionary"), low_word, xconfig->get_lang_name(xconfig, curr_lang));
 		curr_dicts->rem(curr_dicts, low_word);
-		xconfig->save_dicts(xconfig, curr_lang); 
+		xconfig->save_dicts(xconfig, curr_lang);
 	}
 
 	struct _list_char *new_dicts = xconfig->languages[new_lang].dicts;
 	if (!new_dicts->exist(new_dicts, low_word, BY_PLAIN))
 	{
-		log_message(DEBUG, "Add word '%s' in %s dictionary", low_word, xconfig->get_lang_name(xconfig, new_lang));
+		log_message(DEBUG, _("Add word '%s' in %s dictionary"), low_word, xconfig->get_lang_name(xconfig, new_lang));
 		new_dicts->add(new_dicts, low_word);
-		xconfig->save_dicts(xconfig, new_lang); 
+		xconfig->save_dicts(xconfig, new_lang);
 	}
 
 	free(low_word);
@@ -746,7 +998,7 @@ static void xprogram_uninit(struct _xprogram *p)
 
 	free(p);
 
-	log_message(DEBUG, "Program is freed");
+	log_message(DEBUG, _("Program is freed"));
 }
 
 struct _xprogram* xprogram_init(void)
@@ -763,14 +1015,14 @@ struct _xprogram* xprogram_init(void)
 	}
 
 	p->modifier_mask		= NO_MODIFIER_MASK;
-	
+
 	p->event			= xevent_init();		// X Event processor
 	p->focus			= xfocus_init();		// X Input Focus and Pointer processor
 	p->string			= xstring_init();		// Input string buffer
 	p->cursor			= xcursor_init();
-	
+
 	// Function mapping
-	p->uninit			= xprogram_uninit; 
+	p->uninit			= xprogram_uninit;
 	p->cursor_update		= xprogram_cursor_update;
 	p->layout_update		= xprogram_layout_update;
 	p->update			= xprogram_update;
@@ -778,12 +1030,17 @@ struct _xprogram* xprogram_init(void)
 	p->process_input		= xprogram_process_input;
 	p->perform_auto_action		= xprogram_perform_auto_action;
 	p->perform_manual_action	= xprogram_perform_manual_action;
-	p->check_last_word		= xprogram_check_last_word;
+	p->perform_user_action		= xprogram_perform_user_action;
+	p->check_lang_last_word		= xprogram_check_lang_last_word;
+	p->check_caps_last_word		= xprogram_check_caps_last_word;
+	p->check_tcl_last_word		= xprogram_check_tcl_last_word;
 	p->change_word			= xprogram_change_word;
 	p->add_word_to_dict		= xprogram_add_word_to_dict;
-	p->process_selection		= xprogram_process_selection;
+	p->process_selection_notify	= xprogram_process_selection_notify;
 	p->change_lang			= xprogram_change_lang;
+	p->change_incidental_caps = xprogram_change_incidental_caps;
+	p->change_two_capital_letter = xprogram_change_two_capital_letter;
 	p->send_string_silent		= xprogram_send_string_silent;
 
 	return p;
-}
+}
\ No newline at end of file
