@@ -35,8 +35,8 @@ extern struct _xwindow *main_window;
 char* get_selected_text(XSelectionEvent *event)
 {
 	if (event->property == None)
-        {
-		log_message(DEBUG, "Convert to selection target return None answer");
+	{
+		log_message(DEBUG, _("Convert to selection target return None answer"));
 		return NULL;
 	}
 
@@ -48,13 +48,13 @@ char* get_selected_text(XSelectionEvent *event)
 	XGetWindowProperty(main_window->display, main_window->window, event->property, 0, 0, 0, AnyPropertyType, &type, &format, &len, &bytes_left, &data);
 	if (bytes_left == 0)
 	{
-		log_message(DEBUG, "Selected text length is 0");
+		log_message(DEBUG, _("Selected text length is 0"));
 		return NULL;
 	}
 
 	if (XGetWindowProperty(main_window->display, main_window->window, event->property, 0, bytes_left, 0, AnyPropertyType, &type,&format, &len, &dummy, &data) != Success)
 	{
-		log_message(ERROR, "Failed to get selected text data");
+		log_message(ERROR, _("Failed to get selected text data"));
 		return NULL;
 	}
 
@@ -63,11 +63,19 @@ char* get_selected_text(XSelectionEvent *event)
 
 void on_selection_converted(void)
 {
-	XSetSelectionOwner(main_window->display, XA_PRIMARY, None, CurrentTime);
+	Atom selection = XInternAtom(main_window->display, "PRIMARY", FALSE);
+	XSetSelectionOwner(main_window->display, selection, None, CurrentTime);
 }
 
-void do_selection_request(void)
+void do_selection_notify(void)
 {
 	Atom target = XInternAtom(main_window->display, "UTF8_STRING", FALSE);
-	XConvertSelection(main_window->display, XA_PRIMARY, target, None, main_window->window, CurrentTime);
+	Atom selection = XInternAtom(main_window->display, "PRIMARY", FALSE);
+	
+	int status = XConvertSelection(main_window->display, selection, target, None, main_window->window, CurrentTime);
+	if (status == BadAtom)
+		log_message(ERROR, _("Failed to convert selection with error BadAtom"));
+	else if (status == BadWindow)
+		log_message(ERROR, _("Failed to convert selection with error BadWindow"));
 }
+
