@@ -22,5 +22,5 @@
 
 char* get_selected_text(XSelectionEvent *event);
 void  on_selection_converted(void);
-void  do_selection_request(void);
+void  do_selection_notify(void);
 #endif /* _XSELECTION_H_ */
