@@ -7,13 +7,13 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  (c) XNeur Team 2006
+ *  Copyright (C) 2006-2008 XNeur Team
  *
  */
 
@@ -21,9 +21,10 @@
 #define _XSWITCHLANG_H_
 
 int  get_active_keyboard_group(void);
-int  check_keyboard_groups(void);
 int  get_cur_lang(void);
 void switch_lang(int new_lang);
-int  check_xkb_extension(void);
+void switch_group(int new_group);
+int  get_keyboard_groups_count(void);
+int  print_keyboard_groups(void);
 
 #endif /* _XSWITCHLANG_H_ */
