@@ -7,13 +7,13 @@
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- *  (c) XNeur Team 2006
+ *  Copyright (C) 2006-2008 XNeur Team
  *
  */
 
@@ -29,6 +29,11 @@ struct _xwindow
 	Display *display;
 	Window window;
 
+	Window flag_window;
+	
+	Atom close_atom;
+	Atom protocols_atom;
+
 	int  (*create) (struct _xwindow *p);
 	void (*destroy) (struct _xwindow *p);
 	void (*move_window) (struct _xwindow *p, int x, int y);
